package version7_3_simu.platform;

import java.io.FileWriter;
import java.io.PrintWriter;

import version7_3_simu.Interface.*;
import version7_3_simu.display.*;
import version7_3_simu.environment.*;
import version7_3_simu.sensorySystem.*;
import version7_3_simu.spaceMemory.*;
import version7_3_simu.Main;

/**
 * whole learning mechanism
 * This class is used as a bridge between modules
 * @author simon gay
 */
public class Agent {

	public Action action;
	public Perception sensors;
	public InteractionList interactionList;
	
	public Robot body;
	public Observer observer;
	public SpaceMemory spaceMemory;
	
	//%%%%%%%%%%%%%%%%% specific to robots %%%%%%%%%%%%%%%%%%%%%%%%%%%
	public Webcam webcam;
	public ColorRecognition colors;
	public Calibration calibration;
	//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	public Decision decision;
	
	private Main main;
	
	private int ident;
	
	public Agent(int id, Main m){

		action=new Action(this);
		sensors=new Perception(this);
		interactionList=new InteractionList();
		
		body=new Robot();
		decision=new Decision(this);
		observer=new Observer(this);
		spaceMemory=new SpaceMemory(this);
		
		//%%%%%%%%%%%%%%%%% specific to robots %%%%%%%%%%%%%%%%%%%%%%%%%%%
		webcam=new Webcam();
		colors=new ColorRecognition(this);
		calibration=new Calibration(this);
		try{
			Thread.currentThread();
			Thread.sleep(3000);
		}
		catch(Exception ie){}
		webcam.setImage();
		
		try{
			PrintWriter file  = new PrintWriter(new FileWriter(System.getProperty("user.home")+"/Ernest_simulator/enacted.txt"));
			file.print("");
			file.close();
		}catch(Exception e){System.out.println(e);}
		//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

		ident=id;
		
		main=m;
		
	}
	
	// enact a simulation substep
	public void act(){
		action.act();
	}
	
	public boolean isStopped(){
		return action.isStopped();
	}
	
	public int getIdent(){
		return ident;
	}
	
	public Main getMain(){
		return main;
	}
	
	// initialize the set of panels related to the agent
	public void setDisplay(){
		
		boolean efficiency=true;
		boolean spaceMemory=true;
		boolean prediction=true;
		boolean camera=true;
		boolean map=true;
		
		int size;
		int i;
		boolean found;

		//////////////////////
		if (efficiency){
			size=main.display.nbFrame();
			i=0;
			found=false; 
			while (i<size && !found){
				if (main.display.get(i).getClass().getName().equals("version7_3_simu.display.EfficiencyFrame")) found=true;
				i++;
			}
			if (!found) main.display.addFrame(new EfficiencyFrame(this));
			else        
				((EfficiencyFrame)main.display.get(i-1)).setAgent(this);
		}
		
		//////////////////////
		if (spaceMemory){
			size=main.display.nbFrame();
			i=0;
			found=false; 
			while (i<size && !found){
				if (main.display.get(i).getClass().getName().equals("version7_3_simu.display.SpaceFrame")) found=true;
				i++;
			}
			if (!found) main.display.addFrame(new SpaceFrame(this));
			else
				((SpaceFrame)main.display.get(i-1)).setAgent(this);
		}
		
		//////////////////////
		if (prediction){
			size=main.display.nbFrame();
			i=0;
			found=false;
			while (i<size && !found){
				if (main.display.get(i).getClass().getName().equals("version7_3_simu.display.PredictionFrame")) found=true;
				i++;
			}
			if (!found) main.display.addFrame(new PredictionFrame(this));
			else        
				((PredictionFrame)main.display.get(i-1)).setAgent(this);
		}

		//////////////////////
		if (camera){
			size=main.display.nbFrame();
			i=0;
			found=false;
			while (i<size && !found){
				if (main.display.get(i).getClass().getName().equals("version7_3_simu.display.CamFrame")) found=true;
				i++;
			}
			if (!found) main.display.addFrame(new CamFrame(this));
			else        
				((CamFrame)main.display.get(i-1)).setAgent(this);
		}

		//////////////////////
		if (map){
			size=main.display.nbFrame();
			i=0;
			found=false;
			while (i<size && !found){
				if (main.display.get(i).getClass().getName().equals("version7_3_simu.display.MapFrame")) found=true;
				i++;
			}
			if (!found) main.display.addFrame(new MapFrame(this));
			else        
				((MapFrame)main.display.get(i-1)).setAgent(this);
		}
	}

	// save modules data
	public void save() {
		spaceMemory.save();
	}
	
	// save images from displayers
	public void saveImage(String path){
		main.saveImage(path);
	}
	
}
